"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  checkHeaders: () => checkHeaders,
  checkIfUuid: () => checkIfUuid,
  processFile: () => processFile
});
module.exports = __toCommonJS(src_exports);
var import_node_path = __toESM(require("path"));
var import_node_fs = __toESM(require("fs"));
var import_busboy = __toESM(require("busboy"));
var import_fs_jetpack = __toESM(require("fs-jetpack"));
var import_uuid = require("uuid");
var DEBUG = false;
var checkIfUuid = /* @__PURE__ */ __name((headers) => {
  if (!headers["chibi-uuid"])
    return false;
  if (typeof headers["chibi-uuid"] !== "string")
    throw new Error("chibi-uuid is not a string");
  if (headers["chibi-uuid"].length !== 36)
    throw new Error("chibi-uuid does not meet the length criteria");
  if (!/[\da-f]{8}-[\da-f]{4}-4[\da-f]{3}-[89ab][\da-f]{3}-[\da-f]{12}/.test(headers["chibi-uuid"]))
    throw new Error("chibi-uuid is not a valid uuid");
  return true;
}, "checkIfUuid");
var checkHeaders = /* @__PURE__ */ __name((headers) => {
  return headers["chibi-chunk-number"] && headers["chibi-chunks-total"] && /^\d+$/.test(headers["chibi-chunk-number"]) && /^\d+$/.test(headers["chibi-chunks-total"]) || false;
}, "checkHeaders");
var isBiggerThanMaxSize = /* @__PURE__ */ __name((maxFileSize, maxChunkSize, totalChunks) => {
  return maxChunkSize * totalChunks > maxFileSize;
}, "isBiggerThanMaxSize");
var joinChunks = /* @__PURE__ */ __name(async (finalFile, dirPath, totalChunks) => {
  if (DEBUG)
    console.log("[ChibiUploader] Attempting to join chunks");
  const writeStream = (0, import_node_fs.createWriteStream)(finalFile);
  let chunkCount = 1;
  return new Promise((resolve, reject) => {
    const pipeChunk = /* @__PURE__ */ __name(async () => {
      try {
        if (DEBUG)
          console.log("[ChibiUploader] Chunk file:", import_node_path.default.join(dirPath, chunkCount.toString()));
        const readStream = (0, import_node_fs.createReadStream)(import_node_path.default.join(dirPath, chunkCount.toString()));
        readStream.on("error", () => {
          reject(new Error("Error reading chunk"));
        });
        readStream.on("data", (chunk) => {
          if (!chunk) {
            reject(new Error("Chunk is null"));
          }
          writeStream.write(chunk);
        });
        readStream.on("end", async () => {
          chunkCount++;
          if (chunkCount <= totalChunks) {
            await pipeChunk();
          } else {
            writeStream.end();
            if (DEBUG)
              console.log("[ChibiUploader] All chunks joined, deleting temp folder", dirPath);
            await import_fs_jetpack.default.removeAsync(dirPath);
            resolve({ path: finalFile });
          }
        });
      } catch (error) {
        console.error(error);
        return reject;
      }
    }, "pipeChunk");
    void pipeChunk();
  });
}, "joinChunks");
var handleFile = /* @__PURE__ */ __name((tmpDir, headers, fileStream, uuid, metadata) => {
  const filePath = import_node_path.default.join(tmpDir, `${uuid}${import_node_path.default.extname(metadata.name)}`);
  const writeFile = new Promise((resolve, reject) => {
    const writeStream = import_node_fs.default.createWriteStream(filePath, { emitClose: true });
    writeStream.on("error", (err) => {
      fileStream.resume();
      reject(err);
    });
    writeStream.on("close", () => {
      resolve();
    });
    fileStream.pipe(writeStream);
  });
  return async (callback) => {
    try {
      await writeFile;
      return callback(null, { path: filePath });
    } catch (error) {
      return callback(error);
    }
  };
}, "handleFile");
var handleFileWithChunks = /* @__PURE__ */ __name((tmpDir, headers, fileStream, metadata) => {
  const filePath = import_node_path.default.join(tmpDir, `${headers["chibi-uuid"]}`);
  const dirPath = import_node_path.default.join(tmpDir, `${headers["chibi-uuid"]}_tmp`);
  const chunkPath = import_node_path.default.join(dirPath, String(headers["chibi-chunk-number"]));
  const chunkCount = Number(headers["chibi-chunk-number"]);
  const totalChunks = Number(headers["chibi-chunks-total"]);
  let customError;
  const writeFile = new Promise((resolve, reject) => {
    const writeStream = import_node_fs.default.createWriteStream(chunkPath, { emitClose: true });
    writeStream.on("error", (err) => {
      fileStream.resume();
      reject(err);
    });
    writeStream.on("close", async () => {
      if (chunkCount === totalChunks) {
        try {
          await joinChunks(`${filePath}${import_node_path.default.extname(metadata.name)}`, dirPath, totalChunks);
        } catch (error) {
          console.error(error);
        }
        resolve(true);
      }
      resolve(false);
    });
    fileStream.pipe(writeStream);
  });
  import_fs_jetpack.default.dir(dirPath);
  if (chunkCount < 0 || chunkCount > totalChunks) {
    customError = new Error("Chunk is out of range");
    fileStream.resume();
  }
  return async (callback) => {
    try {
      const joined = await writeFile;
      if (joined)
        return callback(null, filePath);
      return callback(null, false);
    } catch (error) {
      return callback(customError ?? error);
    }
  };
}, "handleFileWithChunks");
var processFile = /* @__PURE__ */ __name(async (req, options) => {
  if (options.debug) {
    DEBUG = true;
  }
  await import_fs_jetpack.default.dirAsync(options.destination);
  if (DEBUG)
    console.log("");
  if (DEBUG)
    console.log("[ChibiUploader] Received new file");
  if (DEBUG)
    console.log("[ChibiUploader] maxFileSize:", options.maxFileSize);
  if (DEBUG)
    console.log("[ChibiUploader] maxChunkSize:", options.maxChunkSize);
  return new Promise((resolve, reject) => {
    const usingChunks = checkIfUuid(req.headers);
    if (usingChunks && !checkHeaders(req.headers)) {
      reject(new Error("Invalid headers"));
      return;
    }
    if (isBiggerThanMaxSize(options.maxFileSize, options.maxChunkSize, Number(req.headers["chibi-chunks-total"]))) {
      reject(new Error("Chunked upload is above size limit"));
      return;
    }
    let uuid;
    if (usingChunks) {
      if (DEBUG)
        console.log("[ChibiUploader] Type: Chunked upload");
      if (DEBUG)
        console.log("[ChibiUploader] UUID:", req.headers["chibi-uuid"]);
      if (DEBUG)
        console.log(
          `[ChibiUploader] Chunk number: ${req.headers["chibi-chunk-number"]}/${req.headers["chibi-chunks-total"]}`
        );
    } else {
      uuid = (0, import_uuid.v4)();
      if (DEBUG)
        console.log("[ChibiUploader] Type: Single file upload");
      if (DEBUG)
        console.log("[ChibiUploader] UUID:", uuid);
    }
    try {
      let fileStatus;
      let reachedFileSizeLimit = false;
      const metadata = {};
      const busboy = (0, import_busboy.default)({
        headers: req.headers,
        limits: {
          files: 1,
          // We add an extra 1024 bytes to the maxChunkSize to account for the metadata
          fileSize: options.maxChunkSize + 1024
        },
        defParamCharset: "utf8"
      });
      busboy.on("file", (fieldname, fileStream, info) => {
        if (DEBUG && metadata.name)
          console.log(`[ChibiUploader] Name:`, metadata.name);
        fileStream.on("limit", () => {
          reachedFileSizeLimit = true;
          fileStream.resume();
        });
        if (usingChunks) {
          fileStatus = handleFileWithChunks(options.destination, req.headers, fileStream, metadata);
        } else {
          metadata.name = info.filename;
          metadata.type = info.mimeType;
          fileStatus = handleFile(options.destination, req.headers, fileStream, uuid, metadata);
        }
      });
      busboy.on("field", (key, val) => {
        metadata[key] = val;
      });
      busboy.on("finish", async () => {
        if (reachedFileSizeLimit) {
          if (usingChunks) {
            if (DEBUG)
              console.log("[ChibiUploader] Deleting chunk folder since one of the chunks is too big");
            await import_fs_jetpack.default.removeAsync(import_node_path.default.join(options.destination, `${req.headers["chibi-uuid"]}_tmp`));
            reject(new Error("Chunk is too big"));
          } else {
            if (DEBUG)
              console.log("[ChibiUploader] Deleting file since it is too big");
            await import_fs_jetpack.default.removeAsync(import_node_path.default.join(options.destination, uuid));
            reject(new Error("File is too big"));
          }
          return;
        }
        fileStatus?.(async (fileErr, resultPromise) => {
          if (fileErr) {
            reject(fileErr);
            return;
          }
          if (DEBUG)
            console.log("[ChibiUploader] Done:", metadata.name);
          if (usingChunks) {
            let filePath;
            if (resultPromise) {
              filePath = `${resultPromise}${import_node_path.default.extname(metadata.name)}`;
              if (DEBUG)
                console.log("[ChibiUploader] Filename:", filePath);
            }
            resolve({
              isChunkedUpload: true,
              ready: Boolean(resultPromise),
              path: resultPromise ? filePath : void 0,
              metadata
            });
            return;
          }
          if (DEBUG)
            console.log("[ChibiUploader] Filename:", resultPromise.path);
          try {
            const stat = await import_fs_jetpack.default.inspectAsync(resultPromise.path);
            metadata.size = stat?.size.toString() ?? "0";
          } catch {
            metadata.size = "0";
          }
          resolve({
            isChunkedUpload: false,
            path: resultPromise.path,
            metadata
          });
        });
      });
      req.pipe(busboy);
    } catch (error) {
      reject(error);
      console.log(error);
    }
  });
}, "processFile");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  checkHeaders,
  checkIfUuid,
  processFile
});
//# sourceMappingURL=index.js.map